@@ -1,6 +1,5 @@
 /*
-    <one line to give the library's name and an idea of what it does.>
-    Copyright (C) 2012  Vishesh Handa <me@vhanda.in>
+    Copyright (C) 2012-2014  Vishesh Handa <vhanda@kde.org>
 
     Code largely copied/adapted from KFileMetadataProvider
     Copyright (C) 2010 by Peter Penz <peter.penz@gmx.at>
@@ -24,18 +23,18 @@
 #include "widgetfactory.h"
 #include "tagwidget.h"
 #include "kcommentwidget_p.h"
-#include "kratingwidget.h"
+#include "KRatingWidget"
 
-#include <baloo/filemodifyjob.h>
+#include <KFileMetaData/UserMetaData>
 
-#include <QLabel>
-#include <QTime>
+#include <QtWidgets/QLabel>
+#include <QtCore/QTime>
+#include <QtCore/QUrl>
+#include <QtCore/QLocale>
 
-#include <KUrl>
 #include <KJob>
-#include <KDebug>
-#include <KGlobal>
-#include <KLocale>
+#include <KFormat>
+#include <KLocalizedString>
 
 namespace {
     static QString plainText(const QString& richText)
@@ -75,6 +74,27 @@ WidgetFactory::~WidgetFactory()
 //
 // Widget Creation
 //
+static QString toString(const QVariant& value)
+{
+    switch (value.type()) {
+        case QVariant::Int:
+            return QLocale().toString(value.toInt());
+        case QVariant::Double:
+            return QLocale().toString(value.toDouble());
+
+        case QVariant::List: {
+            QStringList list;
+            for (const QVariant& var : value.toList()) {
+                list << toString(var);
+            }
+
+            return list.join(", ");
+        }
+
+        default:
+            return value.toString();
+    }
+}
 
 QWidget* WidgetFactory::createWidget(const QString& prop, const QVariant& value, QWidget* parent)
 {
@@ -91,35 +111,34 @@ QWidget* WidgetFactory::createWidget(const QString& prop, const QVariant& value,
         widget = createTagWidget( tags, parent );
     }
     else {
+        KFormat form;
         // vHanda: FIXME: Add links! Take m_noLinks into consideration
         QString valueString;
 
-        if (prop == "duration") {
-            QTime time = QTime().addSecs(value.toInt());
-            valueString = KGlobal::locale()->formatTime(time, true, true);
-        }
-        else {
+        if (prop == QStringLiteral("duration")) {
+            valueString = form.formatDuration(value.toInt() * 1000);
+        } else if (prop == QStringLiteral("bitRate")) {
+            valueString = i18nc("@label bitrate (per second)", "%1/s", form.formatByteSize(value.toInt(), 1, KFormat::MetricBinaryDialect));
+        } else if (prop == QLatin1String("originUrl")) {
+            if (m_noLinks) {
+                valueString = value.toString();
+            } else {
+                valueString = QStringLiteral("<a href=\"%1\">%1</a>").arg(value.toString());
+            }
+        } else {
             // Check if Date/DateTime
+
             QDateTime dt = QDateTime::fromString(value.toString(), Qt::ISODate);
             if (dt.isValid()) {
                 QTime time = dt.time();
-                if (!time.hour() && !time.minute() && !time.second())
-                    valueString = KGlobal::locale()->formatDate(dt.date(), KLocale::FancyLongDate);
-                else
-                    valueString = KGlobal::locale()->formatDateTime(dt, KLocale::FancyLongDate);
+                if (!time.hour() && !time.minute() && !time.second()){
+                    valueString = form.formatRelativeDate(dt.date(), QLocale::LongFormat);
+                } else {
+                    valueString = form.formatRelativeDateTime(dt, QLocale::LongFormat);
+                }
             }
             else {
-                switch (value.type()) {
-                case QVariant::Int:
-                    valueString = KGlobal::locale()->formatLong(value.toInt());
-                    break;
-                case QVariant::Double:
-                    valueString = KGlobal::locale()->formatNumber(value.toDouble());
-                    break;
-                default:
-                    valueString = value.toString();
-                    break;
-                }
+                valueString = toString(value);
             }
         }
         widget = createValueWidget(valueString, parent);
@@ -137,10 +156,8 @@ QWidget* WidgetFactory::createTagWidget(const QStringList& tags, QWidget* parent
     tagWidget->setReadyOnly(m_readOnly);
     tagWidget->setSelectedTags(tags);
 
-    connect(tagWidget, SIGNAL(selectionChanged(QStringList)),
-            this, SLOT(slotTagsChanged(QStringList)));
-    connect(tagWidget, SIGNAL(tagClicked(QString)),
-            this, SLOT(slotTagClicked(QString)));
+    connect(tagWidget, &TagWidget::selectionChanged, this, &WidgetFactory::slotTagsChanged);
+    connect(tagWidget, &TagWidget::tagClicked, this, &WidgetFactory::slotTagClicked);
 
     m_tagWidget = tagWidget;
     m_prevTags = tags;
@@ -154,8 +171,7 @@ QWidget* WidgetFactory::createCommentWidget(const QString& comment, QWidget* par
     commentWidget->setText(comment);
     commentWidget->setReadOnly(m_readOnly);
 
-    connect(commentWidget, SIGNAL(commentChanged(QString)),
-            this, SLOT(slotCommentChanged(QString)));
+    connect(commentWidget, &KCommentWidget::commentChanged, this, &WidgetFactory::slotCommentChanged);
 
     m_commentWidget = commentWidget;
 
@@ -172,8 +188,7 @@ QWidget* WidgetFactory::createRatingWidget(int rating, QWidget* parent)
     const QFontMetrics metrics(parent->font());
     ratingWidget->setPixmapSize(metrics.height());
 
-    connect(ratingWidget, SIGNAL(ratingChanged(uint)),
-            this, SLOT(slotRatingChanged(uint)));
+    connect(ratingWidget, static_cast<void (KRatingWidget::*)(uint)>(&KRatingWidget::ratingChanged), this, &WidgetFactory::slotRatingChanged);
 
     m_ratingWidget = ratingWidget;
 
@@ -215,7 +230,7 @@ QWidget* WidgetFactory::createValueWidget(const QString& value, QWidget* parent)
     valueWidget->setWordWrap(true);
     valueWidget->setAlignment(Qt::AlignTop | Qt::AlignLeft);
     valueWidget->setText(m_readOnly ? plainText(value) : value);
-    connect(valueWidget, SIGNAL(linkActivated(QString)), this, SLOT(slotLinkActivated(QString)));
+    connect(valueWidget, &ValueWidget::linkActivated, this, &WidgetFactory::slotLinkActivated);
 
     return valueWidget;
 }
@@ -226,53 +241,61 @@ QWidget* WidgetFactory::createValueWidget(const QString& value, QWidget* parent)
 
 void WidgetFactory::slotCommentChanged(const QString& comment)
 {
-    KJob* job = Baloo::FileModifyJob::modifyUserComment(m_items, comment);
-    startChangeDataJob(job);
+    for (const QString& filePath : m_items) {
+        KFileMetaData::UserMetaData md(filePath);
+        md.setUserComment(comment);
+    }
+    emit dataChangeStarted();
+    emit dataChangeFinished();
 }
 
 void WidgetFactory::slotRatingChanged(uint rating)
 {
-    KJob* job = Baloo::FileModifyJob::modifyRating(m_items, rating);
-    startChangeDataJob(job);
+    for (const QString& filePath : m_items) {
+        KFileMetaData::UserMetaData md(filePath);
+        md.setRating(rating);
+    }
+    emit dataChangeStarted();
+    emit dataChangeFinished();
 }
 
 void WidgetFactory::slotTagsChanged(const QStringList& tags)
 {
     if (m_tagWidget) {
-        KJob* job = Baloo::FileModifyJob::modifyTags(m_items, tags);
+        for (const QString& filePath : m_items) {
+            KFileMetaData::UserMetaData md(filePath);
 
-        // FIXME: vHanda : Remove the tags that are no longer applicable
-        // When multiple tags are selected one doesn't want to loose the old tags
-        // of any of the resources. Unless specifically removed.
-        // QSet<Tag> removedTags = m_prevTags.toSet().subtract( tags.toSet() );
-        // Remove these tags!
+            // When multiple tags are selected one doesn't want to loose the old tags
+            // of any of the resources. Unless specifically removed.
+            QStringList newTags = md.tags() + tags;
+            newTags.removeDuplicates();
+
+            for (const QString& tag : m_prevTags) {
+                if (!tags.contains(tag)) {
+                    newTags.removeAll(tag);
+                }
+            }
+            md.setTags(newTags);
+        }
 
         m_prevTags = tags;
-        startChangeDataJob(job);
+        emit dataChangeStarted();
+        emit dataChangeFinished();
     }
 }
 
-void WidgetFactory::startChangeDataJob(KJob* job)
-{
-    connect(job, SIGNAL(result(KJob*)),
-            this, SIGNAL(dataChangeFinished()));
-
-    emit dataChangeStarted();
-    job->start();
-}
-
 //
 // Notifications
 //
 
 void WidgetFactory::slotLinkActivated(const QString& url)
 {
-    emit urlActivated(KUrl(url));
+    emit urlActivated(QUrl::fromUserInput(url));
 }
 
 void WidgetFactory::slotTagClicked(const QString& tag)
 {
-    KUrl url;
+    QUrl url;
     url.setScheme("tags");
     url.setPath(tag);
 
